#include "effects.h"

#include <num/num.h>
#include "../utilities/fast_trig.h"

#include <cstring>
#include <gint/rtc.h>



#define LENS_WIDTH 120
#define LENS_ZOOM  20


extern bopti_image_t bglens;


static uint32_t lens[LENS_WIDTH][LENS_WIDTH];
int x3 = 16, y3 = 16;
int xd = 1, yd = 1;


void lens_init( bopti_image_t *screen )
{
    image_copy_palette( &bglens, screen, -1 );

    int i, x, y, r, d;

    r = LENS_WIDTH/2;
    d = LENS_ZOOM;

    for (y = 0; y < LENS_WIDTH >> 1; y++)
    {
        for (x = 0; x < LENS_WIDTH >> 1; x++)
        {
            int ix, iy, offset;
            if ((x * x + y * y) < (r * r))
            {
                float shift = d/sqrt(d*d - (x*x + y*y - r*r));
                ix = x * shift - x;
                iy = y * shift - y;
            }
            else
            {
                ix = 0;
                iy = 0;
            }
            offset = (iy * DWIDTH + ix);
            lens[LENS_WIDTH/2 - y][LENS_WIDTH/2 - x] = -offset;
            lens[LENS_WIDTH/2 + y][LENS_WIDTH/2 + x] = offset;
            offset = (-iy * DWIDTH + ix);
            lens[LENS_WIDTH/2 + y][LENS_WIDTH/2 - x] = -offset;
            lens[LENS_WIDTH/2 - y][LENS_WIDTH/2 + x] = offset;
        }
    }  
}


void apply_lens(int ox, int oy, bopti_image_t *screen )
{
    // Apply the lens effect on the picture
    uint8_t* image = (uint8_t*) screen->data;
    uint8_t* back = (uint8_t*) bglens.data;

    image_copy( &bglens, screen, false );

    int x, y, temp, pos;

    for (y = 0; y < LENS_WIDTH; y++)
    {
        temp = (y + oy) * DWIDTH + ox;
        for (x = 0; x < LENS_WIDTH; x++)
        {
            pos = temp + x;
            image[pos] = back[pos + lens[y][x]];
        }
    }
}


void lens_update( bopti_image_t *screen, [[Maybe_unused]] float dt )
{
    apply_lens( x3, y3, screen );

    /* shift the coordinates around */
    x3 += xd;
    y3 += yd;
    if (x3 > (DWIDTH - LENS_WIDTH - 1) || x3 < 1) xd = -xd;
    if (y3 > (MAXHEIGHT - LENS_WIDTH - 1) || y3 < 1) yd = -yd;
}


void lens_render( bopti_image_t *screen )
{
    dimage(0,0,screen);
}


void lens_deinit( bopti_image_t *screen )
{

}