#include "effects.h"

#include <num/num.h>
#include "../utilities/fast_trig.h"

#include <cstring>
#include <gint/rtc.h>


/********************************\
 *   MATRIX EFFECT - MODULE 6   *
 * Specific data and structures *
\********************************/

#define NUMBER_OF_STRIPS 90
#define CHAR_HEIGHT 14
#define CHAR_WIDTH 9

extern font_t font_matrix;

typedef struct
{
    int x;
    int y;
    int speed;
    int len;
    char str[33];
} STRIP;

STRIP *strips;

void init_strip( STRIP* strip )
{
    strip->len = 5 + (rand() % 28);
    strip->x = (rand() % 45);
    strip->y = -1*strip->len*CHAR_HEIGHT;
    strip->speed = 1 + (rand() % 4);

    for( int j=0; j< strips->len; j++ )
        strip->str[j] = 32 + (rand() % 96);
}


void matrix_init( bopti_image_t *screen )
{
    dfont( &font_matrix );
    srand( rtc_ticks() );

    strips = (STRIP *)malloc( NUMBER_OF_STRIPS * sizeof( STRIP ) );

    for( int k=0; k< NUMBER_OF_STRIPS; k++)
        init_strip(strips + k);
}


void matrix_update( bopti_image_t *screen, [[Maybe_unused]] float dt )
{
    for( int k=0; k< NUMBER_OF_STRIPS; k++)
        strips[k].y += strips[k].speed;

    for( int k=0; k< NUMBER_OF_STRIPS; k++)
        if (strips[k].y - strips[k].len*CHAR_HEIGHT>224-CHAR_HEIGHT)
            init_strip( strips + k );
}


void matrix_render( bopti_image_t *screen )
{
    dclear(C_BLACK);

    for( int k=0; k<NUMBER_OF_STRIPS; k++)
        for( int j=0; j<strips[k].len; j++ )
            if ( strips[k].y - j*CHAR_HEIGHT <= MAXHEIGHT )
            {
                if(j==0)      dprint( strips[k].x*CHAR_WIDTH, strips[k].y - j*CHAR_HEIGHT, C_WHITE           , "%c", strips[k].str[j] );
                else if(j==1) dprint( strips[k].x*CHAR_WIDTH, strips[k].y - j*CHAR_HEIGHT, C_RGB(150,255,150), "%c", strips[k].str[j] );
                else if(j==2) dprint( strips[k].x*CHAR_WIDTH, strips[k].y - j*CHAR_HEIGHT, C_RGB(50,255,50)  , "%c", strips[k].str[j] );
                else          dprint( strips[k].x*CHAR_WIDTH, strips[k].y - j*CHAR_HEIGHT, C_GREEN           , "%c", strips[k].str[j] );
            }
}


void matrix_deinit( bopti_image_t *screen )
{
    free( strips );
}